import java.util.Random; 
import java.util.Calendar;
import java.awt.Color;
import processing.core.PApplet;

class SoundWorm
{
  PApplet p;
  SoundBox sbox;
  int x;
  int y;
 
  int hours_color;
  int minutes_color;
  int seconds_color;  
  
  SoundWorm(PApplet p, int x, int y, SoundBox sbox)
  {
    this.p = p;
    this.sbox = sbox;
    this.x = x;
    this.y = y;
  }
  
  void randomizeColors()
  {
    minutes_color = p.color(0,0,0);
    hours_color = p.color(0,0,0);
    seconds_color = p.color(0,0,0);
    
    Random r = new Random();
    int hand = r.nextInt(3);
    switch(hand) {
      case 0: 
        hours_color = p.color(r.nextInt(256),r.nextInt(256),r.nextInt(256));
        break;
      case 1:
        minutes_color = p.color(r.nextInt(256),r.nextInt(256),r.nextInt(256));
        break;
      case 2:
      default:
        seconds_color = p.color(r.nextInt(256),r.nextInt(256),r.nextInt(256));
        break;
    }
  }
  
  void display()
  {    
    //get current time
    Calendar rightNow = Calendar.getInstance();
    int seconds = rightNow.get(Calendar.SECOND);
    int minutes = rightNow.get(Calendar.MINUTE);
    int hours = rightNow.get(Calendar.HOUR); 
    boolean isPM = (rightNow.get(Calendar.AM_PM) == Calendar.PM);
    
     //set maximum arm attributes
    int max_arm_width = 2;
    int max_arm_length = p.min(p.width, p.height) / 3; 
    int max_arm_height = 30;       
  
    //draw circle
    p.pushMatrix();
    p.translate(x, y);
    p.noFill();    
    p.stroke(0, 0, 0);    
    p.smooth();
    if (isPM)
    {
      p.strokeWeight(10);
      p.ellipse(0, 0, 2 * (max_arm_length + max_arm_width + 10), 2 * (max_arm_length + max_arm_width + 10));    
    }
    else
    {
      p.strokeWeight(2);
      p.ellipse(0, 0, 2 * (max_arm_length + max_arm_width + 10) + 4, 2 * (max_arm_length + max_arm_width + 10) + 4);
      p.ellipse(0, 0, 2 * (max_arm_length + max_arm_width + 10) - 4, 2 * (max_arm_length + max_arm_width + 10) - 4);
    }
    p.popMatrix();    

    int interp;
    int buffer_length;
    
    if (!sbox.isActive())
    {
      interp = 0;
      buffer_length = Integer.MAX_VALUE;
    }
    else
    {
       interp = (int) p.max(0,(((p.millis()- sbox.audio_stream.bufferStartTime) / (float)sbox.audio_stream.duration) * sbox.audio_stream.size));
       buffer_length = sbox.audio_stream.buffer2.length;
    }
    
    // draw seconds arm  
    float seconds_angle = (((float)seconds / 60) * 360) - 90;
    float seconds_arm_length = max_arm_length;    
    float seconds_arm_width = 1;
    drawArm(buffer_length, interp, seconds_color, seconds_angle, seconds_arm_length, seconds_arm_width, max_arm_height);
  
    //draw minutes arm
    float minutes_angle = (((float)minutes / 60) * 360) - 90;
    float minutes_arm_length = max_arm_length * .5f;
    float minutes_arm_width = 3;
    drawArm(buffer_length, interp, minutes_color, minutes_angle, minutes_arm_length, minutes_arm_width, max_arm_height);

    //draw hours arm
    float hours_angle = (((float)hours / 12) * 360) - 90;    
    float hours_arm_length = max_arm_length * .3f;
    float hours_arm_width = 5;
    drawArm(buffer_length, interp, hours_color, hours_angle, hours_arm_length, hours_arm_width, max_arm_height);   
  }
  
  void drawArm(int buffer_length, int interp, int arm_color, float arm_angle, float arm_length, float arm_width, float max_arm_height) 
  {
    p.smooth();
    p.noFill();
    p.pushMatrix();    
    p.translate(x, y);
    p.rotate(p.radians(arm_angle));   
    p.stroke(arm_color); 
    p.strokeWeight(arm_width); 
    
    p.beginShape();
    p.curveVertex(0, 0);
    p.curveVertex(0, 0);
    int amplitude = 0;
    int maxx = 0;
    for ( int xx = 0; xx < arm_length; xx++) 
    {    
      if (sbox.isActive())
      {
        if ( xx + interp < buffer_length) 
        {     
          amplitude = (int)((sbox.audio_stream.buffer2[xx + interp] * max_arm_height) * 2);              
          p.curveVertex(xx, amplitude);
          maxx = xx;        
        }
        else
        {
           break;
        }
      }
      else
      {
          amplitude = 0;              
          p.curveVertex(xx, amplitude);
          maxx = xx;   
      }   
    }
    p.curveVertex(maxx, 0);
    p.curveVertex(maxx, 0);  
    p.endShape();
    p.fill(0,0,0);  
    p.ellipse(maxx, 0, 10 , 10);  
    p.popMatrix();
  }
  
  void update()
  {  
    //moveTowards(mouseX, mouseY);
    //rotateTowards(mouseX, mouseY);    
    
    if (x < 0)
    {
      x = 0;
    }
    
    if (x > p.width)
    {
       x = p.width;
    }
    
    if (y < 0)
    {
      y = 0;
    }
    
    if (y > p.height)
    {
      y = p.height;
    }    
  }
  
  void moveTowards(int t_x, int t_y)
  {
    if (x < t_x)
    {
      x++;
    }
    if (x > t_x)
    {
      x--;
    }
    
    if (y < t_y)
    {
      y++;
    }
    
    if(y > t_y)
    {
      y--;
    }
  }

  float rotateTowards(int t_x, int t_y, float angle)
  {
    if (t_x == x && t_y == y)
    {
      return angle;
    }

    float t_angle = p.degrees(p.atan2(t_y - y, t_x - x));
    if (t_angle < 0)
    {
      t_angle = 360 + t_angle;
    }
    
    t_angle -= 180;

    float angle_dist = p.abs(t_angle - angle);
    int rotate_speed = 5;   

    if (angle_dist > rotate_speed)
    {
      if (angle > t_angle)
      {
        if (angle_dist < 180)
        {
          angle -= rotate_speed;
        }
        else
        {
          angle += rotate_speed;
          angle %= 360;
        }                
      }
      else if (angle < t_angle)
      {
          if (angle_dist < 180)
          {
            angle += rotate_speed;
          }
          else
          {
            angle -= rotate_speed;
            angle %= 360;
            if (angle < 0)
            {
              angle += 360;
            }
          }
      }
    }
    
    return angle;     
  }
}
